// Cardboard Piano
// Written by Swapnil Verma - 1 sep 2016
import ddf.minim.*;
import processing.serial.*;

Serial myport;                             // Create object from Serial class 
int val;                                 // Data received from the serial port

PFont f;

PianoKey[] pianoKeys = new PianoKey[13];
int highIndex; // for storing highest index of pianoKeys index

Minim minim;
AudioPlayer song;

class PianoKey {
  String soundFile;  // name of file to play when this note played
  char button;     // used for playing notes from the cardboard piano

  PianoKey(String wavefile, char buttonValue) {
    soundFile = wavefile;
    button = buttonValue;
  }

  void playNote() {    
    // play the sound file from data directory
    song = minim.loadFile(soundFile);
    song.play();
  }
}

void setup() {
  size(400, 200);
  background(255);
  f = createFont("Arial", 1, false);
  frameRate(10);
  // Open the port that the board is connected to and use the same speed (9600 bps) 
  //myport = new Serial(this, "/dev/ttyACM0", 9600);
  minim = new Minim(this);

  // Octave 4 - starting at Left C 
  pianoKeys[0] = new PianoKey("c60.mp3", 'c'); // q // c
  pianoKeys[1] = new PianoKey("c#61.mp3", 'd'); // 2  // d
  pianoKeys[2] = new PianoKey("d62.mp3", '1'); // w // 1
  pianoKeys[3] = new PianoKey("d#63.mp3", '5'); // 3 // 5
  pianoKeys[4] = new PianoKey("e64.mp3", '6'); // e // 6
  pianoKeys[5] = new PianoKey("f65.mp3", '7'); // r // 7
  pianoKeys[6] = new PianoKey("f#66.mp3", '8'); // 5 // 8
  pianoKeys[7] = new PianoKey("g67.mp3", '9'); // t // 9
  pianoKeys[8] = new PianoKey("g#68.mp3", 'a'); // 6  //a
  pianoKeys[9] = new PianoKey("a69.mp3", 'b'); // y // b
  pianoKeys[10] = new PianoKey("a#70.mp3", '2'); // 7 // 2
  pianoKeys[11] = new PianoKey("b71.mp3", '3'); // u // 3

  // Octave 5 - rightmost notes in the image
  pianoKeys[12] = new PianoKey("c72.mp3", '4'); // i // 4
  highIndex = 12;
}

void draw() {
  delay(500);
  pianoGUI();
  Text();
  if (myport.available() > 0) {
    val = myport.read();
    println(val);
    for (int i = 0; i <= highIndex; i++) {
      if (pianoKeys[i].button == val) {
        pianoKeys[i].playNote();
      }
    }
    stroke(2);
    if (val == 'c') { // for c
      fill(255, 0, 0);
      rect(0, 15, 50, 184); // C
      fill(0);
      rect(35, 15, 30, 135); // C#
    } else if (val == 'd') {
      fill(255, 0, 0);
      rect(35, 15, 30, 135); // C#
    } else if (val == '1') {
      fill(255, 0, 0);
      rect(50, 15, 50, 184); // D
      fill(0);
      rect(35, 15, 30, 135); // C#
      rect(85, 15, 30, 135); // D#
    } else if (val == '5') {
      fill(255, 0, 0);
      rect(85, 15, 30, 135); // D#
    } else if (val == '6') {
      fill(255, 0, 0);
      rect(100, 15, 50, 184); // E
      fill(0);
      rect(85, 15, 30, 135); // D#
    } else if (val == '7') {
      fill(255, 0, 0);
      rect(150, 15, 50, 184); // F
      fill(0);
      rect(185, 15, 30, 135); // F#
    } else if (val == '8') {
      fill(255, 0, 0);
      rect(185, 15, 30, 135); // F#
    } else if (val == '9') {
      fill(255, 0, 0);
      rect(200, 15, 50, 184); // G
      fill(0);
      rect(185, 15, 30, 135); // F#
      rect(235, 15, 30, 135); // G#
    } else if (val == 'a') {
      fill(255, 0, 0);
      rect(235, 15, 30, 135); // G#
    } else if (val == 'b') {
      fill(255, 0, 0);
      rect(250, 15, 50, 184); // A
      fill(0);
      rect(235, 15, 30, 135); // G#
      rect(285, 15, 30, 135); // A#
    } else if (val == '2') {
      fill(255, 0, 0);
      rect(285, 15, 30, 135); // A#
    } else if (val == '3') {
      fill(255, 0, 0);
      rect(300, 15, 50, 184); // B
      fill(0);
      rect(285, 15, 30, 135); // A#
    } else if (val == '4') {
      fill(255, 0, 0);
      rect(350, 15, 49, 184); // C
      fill(0);
      rect(385, 15, 15, 135); // Dummy key
    }
  }
}

void Text() {
  textFont(f, 14);                 // STEP 3 Specify font to be used
  // Black keys note
  text("C#", 42, 14); 
  text("D#", 92, 14); 
  text("F#", 192, 14); 
  text("G#", 242, 14); 
  text("A#", 292, 14); 
  // Black keys note
  text("C", 20, 184); 
  text("D", 70, 184); 
  text("E", 120, 184); 
  text("F", 170, 184); 
  text("G", 220, 184); 
  text("A", 270, 184); 
  text("B", 320, 184); 
  text("C", 370, 184);
}

void pianoGUI() {
  stroke(2);
  // White Keys
  fill(255);
  rect(0, 15, 50, 184); // C
  rect(50, 15, 50, 184); // D
  rect(100, 15, 50, 184); // E
  rect(150, 15, 50, 184); // F
  rect(200, 15, 50, 184); // G
  rect(250, 15, 50, 184); // A
  rect(300, 15, 50, 184); // B
  rect(350, 15, 49, 184); // C
  // Black Keys
  fill(0);
  rect(35, 15, 30, 135); // C#
  rect(85, 15, 30, 135); // D#
  rect(185, 15, 30, 135); // F#
  rect(235, 15, 30, 135); // G#
  rect(285, 15, 30, 135); // A#
  rect(385, 15, 15, 135); // Dummy key
}